# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import io
import warnings

from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Dict, List, Optional, Tuple, Union, Any

try:
    from typing import Annotated
except ImportError:
    from typing_extensions import Annotated

from pydantic import Field
from typing_extensions import Annotated
from pydantic import StrictInt, StrictStr, field_validator

from typing import List, Optional

from qaseio.models.base_response import BaseResponse
from qaseio.models.bulk200_response import Bulk200Response
from qaseio.models.id_response import IdResponse
from qaseio.models.test_case_create import TestCaseCreate
from qaseio.models.test_case_list_response import TestCaseListResponse
from qaseio.models.test_case_response import TestCaseResponse
from qaseio.models.test_case_update import TestCaseUpdate
from qaseio.models.test_casebulk import TestCasebulk
from qaseio.models.test_caseexternal_issues import TestCaseexternalIssues

from qaseio.api_client import ApiClient
from qaseio.api_response import ApiResponse
from qaseio.rest import RESTResponseType


class CasesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def bulk(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_casebulk: TestCasebulk,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Bulk200Response:
        """Create test cases in bulk.

        This method allows to bulk create new test cases in a project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_casebulk: (required)
        :type test_casebulk: TestCasebulk
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._bulk_serialize(
            code=code,
            test_casebulk=test_casebulk,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Bulk200Response",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def bulk_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_casebulk: TestCasebulk,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Bulk200Response]:
        """Create test cases in bulk.

        This method allows to bulk create new test cases in a project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_casebulk: (required)
        :type test_casebulk: TestCasebulk
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._bulk_serialize(
            code=code,
            test_casebulk=test_casebulk,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Bulk200Response",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def bulk_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_casebulk: TestCasebulk,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create test cases in bulk.

        This method allows to bulk create new test cases in a project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_casebulk: (required)
        :type test_casebulk: TestCasebulk
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._bulk_serialize(
            code=code,
            test_casebulk=test_casebulk,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Bulk200Response",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _bulk_serialize(
        self,
        code,
        test_casebulk,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if test_casebulk is not None:
            _body_params = test_casebulk


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/case/{code}/bulk',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def case_attach_external_issue(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_caseexternal_issues: TestCaseexternalIssues,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> BaseResponse:
        """Attach the external issues to the test cases.


        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_caseexternal_issues: (required)
        :type test_caseexternal_issues: TestCaseexternalIssues
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._case_attach_external_issue_serialize(
            code=code,
            test_caseexternal_issues=test_caseexternal_issues,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def case_attach_external_issue_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_caseexternal_issues: TestCaseexternalIssues,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[BaseResponse]:
        """Attach the external issues to the test cases.


        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_caseexternal_issues: (required)
        :type test_caseexternal_issues: TestCaseexternalIssues
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._case_attach_external_issue_serialize(
            code=code,
            test_caseexternal_issues=test_caseexternal_issues,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def case_attach_external_issue_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_caseexternal_issues: TestCaseexternalIssues,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Attach the external issues to the test cases.


        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_caseexternal_issues: (required)
        :type test_caseexternal_issues: TestCaseexternalIssues
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._case_attach_external_issue_serialize(
            code=code,
            test_caseexternal_issues=test_caseexternal_issues,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _case_attach_external_issue_serialize(
        self,
        code,
        test_caseexternal_issues,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if test_caseexternal_issues is not None:
            _body_params = test_caseexternal_issues


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/case/{code}/external-issue/attach',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def case_detach_external_issue(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_caseexternal_issues: TestCaseexternalIssues,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> BaseResponse:
        """Detach the external issues from the test cases.


        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_caseexternal_issues: (required)
        :type test_caseexternal_issues: TestCaseexternalIssues
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._case_detach_external_issue_serialize(
            code=code,
            test_caseexternal_issues=test_caseexternal_issues,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def case_detach_external_issue_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_caseexternal_issues: TestCaseexternalIssues,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[BaseResponse]:
        """Detach the external issues from the test cases.


        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_caseexternal_issues: (required)
        :type test_caseexternal_issues: TestCaseexternalIssues
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._case_detach_external_issue_serialize(
            code=code,
            test_caseexternal_issues=test_caseexternal_issues,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def case_detach_external_issue_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_caseexternal_issues: TestCaseexternalIssues,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Detach the external issues from the test cases.


        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_caseexternal_issues: (required)
        :type test_caseexternal_issues: TestCaseexternalIssues
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._case_detach_external_issue_serialize(
            code=code,
            test_caseexternal_issues=test_caseexternal_issues,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _case_detach_external_issue_serialize(
        self,
        code,
        test_caseexternal_issues,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if test_caseexternal_issues is not None:
            _body_params = test_caseexternal_issues


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/case/{code}/external-issue/detach',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def create_case(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_case_create: TestCaseCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> IdResponse:
        """Create a new test case.

        This method allows to create a new test case in selected project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_case_create: (required)
        :type test_case_create: TestCaseCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_case_serialize(
            code=code,
            test_case_create=test_case_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_case_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_case_create: TestCaseCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[IdResponse]:
        """Create a new test case.

        This method allows to create a new test case in selected project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_case_create: (required)
        :type test_case_create: TestCaseCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_case_serialize(
            code=code,
            test_case_create=test_case_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_case_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        test_case_create: TestCaseCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create a new test case.

        This method allows to create a new test case in selected project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param test_case_create: (required)
        :type test_case_create: TestCaseCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_case_serialize(
            code=code,
            test_case_create=test_case_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_case_serialize(
        self,
        code,
        test_case_create,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if test_case_create is not None:
            _body_params = test_case_create


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/case/{code}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_case(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> IdResponse:
        """Delete test case.

        This method completely deletes a test case from repository. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_case_serialize(
            code=code,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_case_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[IdResponse]:
        """Delete test case.

        This method completely deletes a test case from repository. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_case_serialize(
            code=code,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_case_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete test case.

        This method completely deletes a test case from repository. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_case_serialize(
            code=code,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_case_serialize(
        self,
        code,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/case/{code}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_case(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TestCaseResponse:
        """Get a specific test case.

        This method allows to retrieve a specific test case. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_case_serialize(
            code=code,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TestCaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_case_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TestCaseResponse]:
        """Get a specific test case.

        This method allows to retrieve a specific test case. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_case_serialize(
            code=code,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TestCaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_case_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get a specific test case.

        This method allows to retrieve a specific test case. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_case_serialize(
            code=code,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TestCaseResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_case_serialize(
        self,
        code,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/case/{code}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_cases(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        search: Annotated[Optional[StrictStr], Field(description="Provide a string that will be used to search by name.")] = None,
        milestone_id: Annotated[Optional[StrictInt], Field(description="ID of milestone.")] = None,
        suite_id: Annotated[Optional[StrictInt], Field(description="ID of test suite.")] = None,
        severity: Annotated[Optional[StrictStr], Field(description="A list of severity values separated by comma. Possible values: undefined, blocker, critical, major, normal, minor, trivial ")] = None,
        priority: Annotated[Optional[StrictStr], Field(description="A list of priority values separated by comma. Possible values: undefined, high, medium, low ")] = None,
        type: Annotated[Optional[StrictStr], Field(description="A list of type values separated by comma. Possible values: other, functional smoke, regression, security, usability, performance, acceptance ")] = None,
        behavior: Annotated[Optional[StrictStr], Field(description="A list of behavior values separated by comma. Possible values: undefined, positive negative, destructive ")] = None,
        automation: Annotated[Optional[StrictStr], Field(description="A list of values separated by comma. Possible values: is-not-automated, automated to-be-automated ")] = None,
        status: Annotated[Optional[StrictStr], Field(description="A list of values separated by comma. Possible values: actual, draft deprecated ")] = None,
        external_issues_type: Annotated[Optional[StrictStr], Field(description="An integration type. ")] = None,
        external_issues_ids: Annotated[Optional[List[StrictStr]], Field(description="A list of issue IDs.")] = None,
        include: Annotated[Optional[StrictStr], Field(description="A list of entities to include in response separated by comma. Possible values: external_issues. ")] = None,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="A number of entities in result set.")] = None,
        offset: Annotated[Optional[Annotated[int, Field(le=100000, strict=True, ge=0)]], Field(description="How many entities should be skipped.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TestCaseListResponse:
        """Get all test cases.

        This method allows to retrieve all test cases stored in selected project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param search: Provide a string that will be used to search by name.
        :type search: str
        :param milestone_id: ID of milestone.
        :type milestone_id: int
        :param suite_id: ID of test suite.
        :type suite_id: int
        :param severity: A list of severity values separated by comma. Possible values: undefined, blocker, critical, major, normal, minor, trivial 
        :type severity: str
        :param priority: A list of priority values separated by comma. Possible values: undefined, high, medium, low 
        :type priority: str
        :param type: A list of type values separated by comma. Possible values: other, functional smoke, regression, security, usability, performance, acceptance 
        :type type: str
        :param behavior: A list of behavior values separated by comma. Possible values: undefined, positive negative, destructive 
        :type behavior: str
        :param automation: A list of values separated by comma. Possible values: is-not-automated, automated to-be-automated 
        :type automation: str
        :param status: A list of values separated by comma. Possible values: actual, draft deprecated 
        :type status: str
        :param external_issues_type: An integration type. 
        :type external_issues_type: str
        :param external_issues_ids: A list of issue IDs.
        :type external_issues_ids: List[str]
        :param include: A list of entities to include in response separated by comma. Possible values: external_issues. 
        :type include: str
        :param limit: A number of entities in result set.
        :type limit: int
        :param offset: How many entities should be skipped.
        :type offset: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_cases_serialize(
            code=code,
            search=search,
            milestone_id=milestone_id,
            suite_id=suite_id,
            severity=severity,
            priority=priority,
            type=type,
            behavior=behavior,
            automation=automation,
            status=status,
            external_issues_type=external_issues_type,
            external_issues_ids=external_issues_ids,
            include=include,
            limit=limit,
            offset=offset,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TestCaseListResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_cases_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        search: Annotated[Optional[StrictStr], Field(description="Provide a string that will be used to search by name.")] = None,
        milestone_id: Annotated[Optional[StrictInt], Field(description="ID of milestone.")] = None,
        suite_id: Annotated[Optional[StrictInt], Field(description="ID of test suite.")] = None,
        severity: Annotated[Optional[StrictStr], Field(description="A list of severity values separated by comma. Possible values: undefined, blocker, critical, major, normal, minor, trivial ")] = None,
        priority: Annotated[Optional[StrictStr], Field(description="A list of priority values separated by comma. Possible values: undefined, high, medium, low ")] = None,
        type: Annotated[Optional[StrictStr], Field(description="A list of type values separated by comma. Possible values: other, functional smoke, regression, security, usability, performance, acceptance ")] = None,
        behavior: Annotated[Optional[StrictStr], Field(description="A list of behavior values separated by comma. Possible values: undefined, positive negative, destructive ")] = None,
        automation: Annotated[Optional[StrictStr], Field(description="A list of values separated by comma. Possible values: is-not-automated, automated to-be-automated ")] = None,
        status: Annotated[Optional[StrictStr], Field(description="A list of values separated by comma. Possible values: actual, draft deprecated ")] = None,
        external_issues_type: Annotated[Optional[StrictStr], Field(description="An integration type. ")] = None,
        external_issues_ids: Annotated[Optional[List[StrictStr]], Field(description="A list of issue IDs.")] = None,
        include: Annotated[Optional[StrictStr], Field(description="A list of entities to include in response separated by comma. Possible values: external_issues. ")] = None,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="A number of entities in result set.")] = None,
        offset: Annotated[Optional[Annotated[int, Field(le=100000, strict=True, ge=0)]], Field(description="How many entities should be skipped.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TestCaseListResponse]:
        """Get all test cases.

        This method allows to retrieve all test cases stored in selected project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param search: Provide a string that will be used to search by name.
        :type search: str
        :param milestone_id: ID of milestone.
        :type milestone_id: int
        :param suite_id: ID of test suite.
        :type suite_id: int
        :param severity: A list of severity values separated by comma. Possible values: undefined, blocker, critical, major, normal, minor, trivial 
        :type severity: str
        :param priority: A list of priority values separated by comma. Possible values: undefined, high, medium, low 
        :type priority: str
        :param type: A list of type values separated by comma. Possible values: other, functional smoke, regression, security, usability, performance, acceptance 
        :type type: str
        :param behavior: A list of behavior values separated by comma. Possible values: undefined, positive negative, destructive 
        :type behavior: str
        :param automation: A list of values separated by comma. Possible values: is-not-automated, automated to-be-automated 
        :type automation: str
        :param status: A list of values separated by comma. Possible values: actual, draft deprecated 
        :type status: str
        :param external_issues_type: An integration type. 
        :type external_issues_type: str
        :param external_issues_ids: A list of issue IDs.
        :type external_issues_ids: List[str]
        :param include: A list of entities to include in response separated by comma. Possible values: external_issues. 
        :type include: str
        :param limit: A number of entities in result set.
        :type limit: int
        :param offset: How many entities should be skipped.
        :type offset: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_cases_serialize(
            code=code,
            search=search,
            milestone_id=milestone_id,
            suite_id=suite_id,
            severity=severity,
            priority=priority,
            type=type,
            behavior=behavior,
            automation=automation,
            status=status,
            external_issues_type=external_issues_type,
            external_issues_ids=external_issues_ids,
            include=include,
            limit=limit,
            offset=offset,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TestCaseListResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_cases_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        search: Annotated[Optional[StrictStr], Field(description="Provide a string that will be used to search by name.")] = None,
        milestone_id: Annotated[Optional[StrictInt], Field(description="ID of milestone.")] = None,
        suite_id: Annotated[Optional[StrictInt], Field(description="ID of test suite.")] = None,
        severity: Annotated[Optional[StrictStr], Field(description="A list of severity values separated by comma. Possible values: undefined, blocker, critical, major, normal, minor, trivial ")] = None,
        priority: Annotated[Optional[StrictStr], Field(description="A list of priority values separated by comma. Possible values: undefined, high, medium, low ")] = None,
        type: Annotated[Optional[StrictStr], Field(description="A list of type values separated by comma. Possible values: other, functional smoke, regression, security, usability, performance, acceptance ")] = None,
        behavior: Annotated[Optional[StrictStr], Field(description="A list of behavior values separated by comma. Possible values: undefined, positive negative, destructive ")] = None,
        automation: Annotated[Optional[StrictStr], Field(description="A list of values separated by comma. Possible values: is-not-automated, automated to-be-automated ")] = None,
        status: Annotated[Optional[StrictStr], Field(description="A list of values separated by comma. Possible values: actual, draft deprecated ")] = None,
        external_issues_type: Annotated[Optional[StrictStr], Field(description="An integration type. ")] = None,
        external_issues_ids: Annotated[Optional[List[StrictStr]], Field(description="A list of issue IDs.")] = None,
        include: Annotated[Optional[StrictStr], Field(description="A list of entities to include in response separated by comma. Possible values: external_issues. ")] = None,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="A number of entities in result set.")] = None,
        offset: Annotated[Optional[Annotated[int, Field(le=100000, strict=True, ge=0)]], Field(description="How many entities should be skipped.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all test cases.

        This method allows to retrieve all test cases stored in selected project. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param search: Provide a string that will be used to search by name.
        :type search: str
        :param milestone_id: ID of milestone.
        :type milestone_id: int
        :param suite_id: ID of test suite.
        :type suite_id: int
        :param severity: A list of severity values separated by comma. Possible values: undefined, blocker, critical, major, normal, minor, trivial 
        :type severity: str
        :param priority: A list of priority values separated by comma. Possible values: undefined, high, medium, low 
        :type priority: str
        :param type: A list of type values separated by comma. Possible values: other, functional smoke, regression, security, usability, performance, acceptance 
        :type type: str
        :param behavior: A list of behavior values separated by comma. Possible values: undefined, positive negative, destructive 
        :type behavior: str
        :param automation: A list of values separated by comma. Possible values: is-not-automated, automated to-be-automated 
        :type automation: str
        :param status: A list of values separated by comma. Possible values: actual, draft deprecated 
        :type status: str
        :param external_issues_type: An integration type. 
        :type external_issues_type: str
        :param external_issues_ids: A list of issue IDs.
        :type external_issues_ids: List[str]
        :param include: A list of entities to include in response separated by comma. Possible values: external_issues. 
        :type include: str
        :param limit: A number of entities in result set.
        :type limit: int
        :param offset: How many entities should be skipped.
        :type offset: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_cases_serialize(
            code=code,
            search=search,
            milestone_id=milestone_id,
            suite_id=suite_id,
            severity=severity,
            priority=priority,
            type=type,
            behavior=behavior,
            automation=automation,
            status=status,
            external_issues_type=external_issues_type,
            external_issues_ids=external_issues_ids,
            include=include,
            limit=limit,
            offset=offset,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TestCaseListResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_cases_serialize(
        self,
        code,
        search,
        milestone_id,
        suite_id,
        severity,
        priority,
        type,
        behavior,
        automation,
        status,
        external_issues_type,
        external_issues_ids,
        include,
        limit,
        offset,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
            'external_issues[ids][]': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        # process the query parameters
        if search is not None:
            
            _query_params.append(('search', search))
            
        if milestone_id is not None:
            
            _query_params.append(('milestone_id', milestone_id))
            
        if suite_id is not None:
            
            _query_params.append(('suite_id', suite_id))
            
        if severity is not None:
            
            _query_params.append(('severity', severity))
            
        if priority is not None:
            
            _query_params.append(('priority', priority))
            
        if type is not None:
            
            _query_params.append(('type', type))
            
        if behavior is not None:
            
            _query_params.append(('behavior', behavior))
            
        if automation is not None:
            
            _query_params.append(('automation', automation))
            
        if status is not None:
            
            _query_params.append(('status', status))
            
        if external_issues_type is not None:
            
            _query_params.append(('external_issues[type]', external_issues_type))
            
        if external_issues_ids is not None:
            
            _query_params.append(('external_issues[ids][]', external_issues_ids))
            
        if include is not None:
            
            _query_params.append(('include', include))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if offset is not None:
            
            _query_params.append(('offset', offset))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/case/{code}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_case(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        test_case_update: TestCaseUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> IdResponse:
        """Update test case.

        This method updates a test case. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param test_case_update: (required)
        :type test_case_update: TestCaseUpdate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_case_serialize(
            code=code,
            id=id,
            test_case_update=test_case_update,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_case_with_http_info(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        test_case_update: TestCaseUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[IdResponse]:
        """Update test case.

        This method updates a test case. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param test_case_update: (required)
        :type test_case_update: TestCaseUpdate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_case_serialize(
            code=code,
            id=id,
            test_case_update=test_case_update,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_case_without_preload_content(
        self,
        code: Annotated[str, Field(min_length=2, strict=True, max_length=10, description="Code of project, where to search entities.")],
        id: Annotated[StrictInt, Field(description="Identifier.")],
        test_case_update: TestCaseUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update test case.

        This method updates a test case. 

        :param code: Code of project, where to search entities. (required)
        :type code: str
        :param id: Identifier. (required)
        :type id: int
        :param test_case_update: (required)
        :type test_case_update: TestCaseUpdate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_case_serialize(
            code=code,
            id=id,
            test_case_update=test_case_update,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IdResponse",
            '400': None,
            '401': None,
            '403': None,
            '404': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_case_serialize(
        self,
        code,
        id,
        test_case_update,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if test_case_update is not None:
            _body_params = test_case_update


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'application/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'TokenAuth'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/case/{code}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.defect_query import DefectQuery

class TestDefectQuery(unittest.TestCase):
    """DefectQuery unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> DefectQuery:
        """Test DefectQuery
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `DefectQuery`
        """
        model = DefectQuery()
        if include_optional:
            return DefectQuery(
                id = 56,
                title = '',
                actual_result = '',
                severity = '',
                status = '',
                milestone_id = 56,
                custom_fields = [
                    qaseio.models.custom_field_value.CustomFieldValue(
                        id = 56, 
                        value = '', )
                    ],
                attachments = [
                    qaseio.models.attachment.Attachment(
                        size = 56, 
                        mime = '', 
                        filename = '', 
                        url = '', )
                    ],
                resolved = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                member_id = 56,
                author_id = 56,
                external_data = '',
                tags = [
                    qaseio.models.tag_value.TagValue(
                        title = '', 
                        internal_id = 56, )
                    ],
                created_at = '2021-12-30T19:23:59Z',
                updated_at = '2021-12-30T19:23:59Z'
            )
        else:
            return DefectQuery(
        )
        """

    def testDefectQuery(self):
        """Test DefectQuery"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.project import Project

class TestProject(unittest.TestCase):
    """Project unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Project:
        """Test Project
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Project`
        """
        model = Project()
        if include_optional:
            return Project(
                title = '',
                code = '',
                counts = qaseio.models.project_counts.Project_counts(
                    cases = 56, 
                    suites = 56, 
                    milestones = 56, 
                    runs = qaseio.models.project_counts_runs.Project_counts_runs(
                        total = 56, 
                        active = 56, ), 
                    defects = qaseio.models.project_counts_defects.Project_counts_defects(
                        total = 56, 
                        open = 56, ), )
            )
        else:
            return Project(
        )
        """

    def testProject(self):
        """Test Project"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.defect_response import DefectResponse

class TestDefectResponse(unittest.TestCase):
    """DefectResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> DefectResponse:
        """Test DefectResponse
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `DefectResponse`
        """
        model = DefectResponse()
        if include_optional:
            return DefectResponse(
                status = True,
                result = qaseio.models.defect.Defect(
                    id = 56, 
                    title = '', 
                    actual_result = '', 
                    severity = '', 
                    status = '', 
                    milestone_id = 56, 
                    custom_fields = [
                        qaseio.models.custom_field_value.CustomFieldValue(
                            id = 56, 
                            value = '', )
                        ], 
                    attachments = [
                        qaseio.models.attachment.Attachment(
                            size = 56, 
                            mime = '', 
                            filename = '', 
                            url = '', )
                        ], 
                    resolved_at = '2021-12-30T19:23:59Z', 
                    member_id = 56, 
                    author_id = 56, 
                    external_data = '', 
                    tags = [
                        qaseio.models.tag_value.TagValue(
                            title = '', 
                            internal_id = 56, )
                        ], 
                    created_at = '2021-12-30T19:23:59Z', 
                    updated_at = '2021-12-30T19:23:59Z', 
                    created = '2021-12-30 19:23:59', 
                    updated = '2021-12-30 19:23:59', )
            )
        else:
            return DefectResponse(
        )
        """

    def testDefectResponse(self):
        """Test DefectResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictInt, StrictStr
from pydantic import Field
from qaseio.models.attachment import Attachment
from qaseio.models.custom_field_value import CustomFieldValue
from qaseio.models.tag_value import TagValue
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class QqlDefect(BaseModel):
    """
    QqlDefect
    """ # noqa: E501
    id: Optional[StrictInt] = None
    title: Optional[StrictStr] = None
    actual_result: Optional[StrictStr] = None
    severity: Optional[StrictStr] = None
    status: Optional[StrictStr] = None
    milestone_id: Optional[StrictInt] = None
    custom_fields: Optional[List[CustomFieldValue]] = None
    attachments: Optional[List[Attachment]] = None
    resolved: Optional[datetime] = None
    member_id: Optional[StrictInt] = Field(default=None, description="Deprecated, use `author_id` instead.")
    author_id: Optional[StrictInt] = None
    external_data: Optional[StrictStr] = None
    tags: Optional[List[TagValue]] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    __properties: ClassVar[List[str]] = ["id", "title", "actual_result", "severity", "status", "milestone_id", "custom_fields", "attachments", "resolved", "member_id", "author_id", "external_data", "tags", "created_at", "updated_at"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of QqlDefect from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in custom_fields (list)
        _items = []
        if self.custom_fields:
            for _item in self.custom_fields:
                if _item:
                    _items.append(_item.to_dict())
            _dict['custom_fields'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in attachments (list)
        _items = []
        if self.attachments:
            for _item in self.attachments:
                if _item:
                    _items.append(_item.to_dict())
            _dict['attachments'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tags (list)
        _items = []
        if self.tags:
            for _item in self.tags:
                if _item:
                    _items.append(_item.to_dict())
            _dict['tags'] = _items
        # set to None if milestone_id (nullable) is None
        # and model_fields_set contains the field
        if self.milestone_id is None and "milestone_id" in self.model_fields_set:
            _dict['milestone_id'] = None

        # set to None if resolved (nullable) is None
        # and model_fields_set contains the field
        if self.resolved is None and "resolved" in self.model_fields_set:
            _dict['resolved'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of QqlDefect from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "title": obj.get("title"),
            "actual_result": obj.get("actual_result"),
            "severity": obj.get("severity"),
            "status": obj.get("status"),
            "milestone_id": obj.get("milestone_id"),
            "custom_fields": [CustomFieldValue.from_dict(_item) for _item in obj.get("custom_fields")] if obj.get("custom_fields") is not None else None,
            "attachments": [Attachment.from_dict(_item) for _item in obj.get("attachments")] if obj.get("attachments") is not None else None,
            "resolved": obj.get("resolved"),
            "member_id": obj.get("member_id"),
            "author_id": obj.get("author_id"),
            "external_data": obj.get("external_data"),
            "tags": [TagValue.from_dict(_item) for _item in obj.get("tags")] if obj.get("tags") is not None else None,
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at")
        })
        return _obj





# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.result_response import ResultResponse

class TestResultResponse(unittest.TestCase):
    """ResultResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ResultResponse:
        """Test ResultResponse
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ResultResponse`
        """
        model = ResultResponse()
        if include_optional:
            return ResultResponse(
                status = True,
                result = qaseio.models.result.Result(
                    hash = '', 
                    comment = '', 
                    stacktrace = '', 
                    run_id = 56, 
                    case_id = 56, 
                    steps = [
                        qaseio.models.test_step_result.TestStepResult(
                            status = 56, 
                            position = 56, 
                            attachments = [
                                qaseio.models.attachment.Attachment(
                                    size = 56, 
                                    mime = '', 
                                    filename = '', 
                                    url = '', )
                                ], )
                        ], 
                    status = '', 
                    is_api_result = True, 
                    time_spent_ms = 56, 
                    end_time = '2021-12-30T19:23:59Z', 
                    attachments = [
                        qaseio.models.attachment.Attachment(
                            size = 56, 
                            mime = '', 
                            filename = '', 
                            url = '', )
                        ], )
            )
        else:
            return ResultResponse(
        )
        """

    def testResultResponse(self):
        """Test ResultResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.result_create_response import ResultCreateResponse

class TestResultCreateResponse(unittest.TestCase):
    """ResultCreateResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ResultCreateResponse:
        """Test ResultCreateResponse
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ResultCreateResponse`
        """
        model = ResultCreateResponse()
        if include_optional:
            return ResultCreateResponse(
                status = True,
                result = qaseio.models.result_create_response_all_of_result.ResultCreateResponse_allOf_result(
                    case_id = 56, 
                    hash = '', )
            )
        else:
            return ResultCreateResponse(
        )
        """

    def testResultCreateResponse(self):
        """Test ResultCreateResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.result_list_response_all_of_result import ResultListResponseAllOfResult

class TestResultListResponseAllOfResult(unittest.TestCase):
    """ResultListResponseAllOfResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ResultListResponseAllOfResult:
        """Test ResultListResponseAllOfResult
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ResultListResponseAllOfResult`
        """
        model = ResultListResponseAllOfResult()
        if include_optional:
            return ResultListResponseAllOfResult(
                total = 56,
                filtered = 56,
                count = 56,
                entities = [
                    qaseio.models.result.Result(
                        hash = '', 
                        comment = '', 
                        stacktrace = '', 
                        run_id = 56, 
                        case_id = 56, 
                        steps = [
                            qaseio.models.test_step_result.TestStepResult(
                                status = 56, 
                                position = 56, 
                                attachments = [
                                    qaseio.models.attachment.Attachment(
                                        size = 56, 
                                        mime = '', 
                                        filename = '', 
                                        url = '', )
                                    ], )
                            ], 
                        status = '', 
                        is_api_result = True, 
                        time_spent_ms = 56, 
                        end_time = '2021-12-30T19:23:59Z', 
                        attachments = [
                            qaseio.models.attachment.Attachment(
                                size = 56, 
                                mime = '', 
                                filename = '', 
                                url = '', )
                            ], )
                    ]
            )
        else:
            return ResultListResponseAllOfResult(
        )
        """

    def testResultListResponseAllOfResult(self):
        """Test ResultListResponseAllOfResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictBool, StrictStr, field_validator
from pydantic import Field
from typing_extensions import Annotated
from qaseio.models.test_step_result_create import TestStepResultCreate
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class ResultUpdate(BaseModel):
    """
    ResultUpdate
    """ # noqa: E501
    status: Optional[StrictStr] = None
    time_ms: Optional[Annotated[int, Field(le=31536000000, strict=True, ge=0)]] = None
    defect: Optional[StrictBool] = None
    attachments: Optional[List[StrictStr]] = None
    stacktrace: Optional[StrictStr] = None
    comment: Optional[StrictStr] = None
    steps: Optional[List[TestStepResultCreate]] = None
    __properties: ClassVar[List[str]] = ["status", "time_ms", "defect", "attachments", "stacktrace", "comment", "steps"]

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('in_progress', 'passed', 'failed', 'blocked', 'skipped'):
            raise ValueError("must be one of enum values ('in_progress', 'passed', 'failed', 'blocked', 'skipped')")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ResultUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in steps (list)
        _items = []
        if self.steps:
            for _item in self.steps:
                if _item:
                    _items.append(_item.to_dict())
            _dict['steps'] = _items
        # set to None if time_ms (nullable) is None
        # and model_fields_set contains the field
        if self.time_ms is None and "time_ms" in self.model_fields_set:
            _dict['time_ms'] = None

        # set to None if defect (nullable) is None
        # and model_fields_set contains the field
        if self.defect is None and "defect" in self.model_fields_set:
            _dict['defect'] = None

        # set to None if attachments (nullable) is None
        # and model_fields_set contains the field
        if self.attachments is None and "attachments" in self.model_fields_set:
            _dict['attachments'] = None

        # set to None if stacktrace (nullable) is None
        # and model_fields_set contains the field
        if self.stacktrace is None and "stacktrace" in self.model_fields_set:
            _dict['stacktrace'] = None

        # set to None if comment (nullable) is None
        # and model_fields_set contains the field
        if self.comment is None and "comment" in self.model_fields_set:
            _dict['comment'] = None

        # set to None if steps (nullable) is None
        # and model_fields_set contains the field
        if self.steps is None and "steps" in self.model_fields_set:
            _dict['steps'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ResultUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "status": obj.get("status"),
            "time_ms": obj.get("time_ms"),
            "defect": obj.get("defect"),
            "attachments": obj.get("attachments"),
            "stacktrace": obj.get("stacktrace"),
            "comment": obj.get("comment"),
            "steps": [TestStepResultCreate.from_dict(_item) for _item in obj.get("steps")] if obj.get("steps") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.result_list_response import ResultListResponse

class TestResultListResponse(unittest.TestCase):
    """ResultListResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ResultListResponse:
        """Test ResultListResponse
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ResultListResponse`
        """
        model = ResultListResponse()
        if include_optional:
            return ResultListResponse(
                status = True,
                result = qaseio.models.result_list_response_all_of_result.ResultListResponse_allOf_result(
                    total = 56, 
                    filtered = 56, 
                    count = 56, 
                    entities = [
                        qaseio.models.result.Result(
                            hash = '', 
                            comment = '', 
                            stacktrace = '', 
                            run_id = 56, 
                            case_id = 56, 
                            steps = [
                                qaseio.models.test_step_result.TestStepResult(
                                    status = 56, 
                                    position = 56, 
                                    attachments = [
                                        qaseio.models.attachment.Attachment(
                                            size = 56, 
                                            mime = '', 
                                            filename = '', 
                                            url = '', )
                                        ], )
                                ], 
                            status = '', 
                            is_api_result = True, 
                            time_spent_ms = 56, 
                            end_time = '2021-12-30T19:23:59Z', 
                            attachments = [
                                qaseio.models.attachment.Attachment(
                                    size = 56, 
                                    mime = '', 
                                    filename = '', 
                                    url = '', )
                                ], )
                        ], )
            )
        else:
            return ResultListResponse(
        )
        """

    def testResultListResponse(self):
        """Test ResultListResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "4.0.0"

# import apis into sdk package
from qaseio.api.attachments_api import AttachmentsApi
from qaseio.api.authors_api import AuthorsApi
from qaseio.api.cases_api import CasesApi
from qaseio.api.custom_fields_api import CustomFieldsApi
from qaseio.api.defects_api import DefectsApi
from qaseio.api.environments_api import EnvironmentsApi
from qaseio.api.milestones_api import MilestonesApi
from qaseio.api.plans_api import PlansApi
from qaseio.api.projects_api import ProjectsApi
from qaseio.api.result_api import ResultApi
from qaseio.api.results_api import ResultsApi
from qaseio.api.runs_api import RunsApi
from qaseio.api.search_api import SearchApi
from qaseio.api.shared_steps_api import SharedStepsApi
from qaseio.api.suites_api import SuitesApi
from qaseio.api.system_fields_api import SystemFieldsApi

# import ApiClient
from qaseio.api_response import ApiResponse
from qaseio.api_client import ApiClient
from qaseio.configuration import Configuration
from qaseio.exceptions import OpenApiException
from qaseio.exceptions import ApiTypeError
from qaseio.exceptions import ApiValueError
from qaseio.exceptions import ApiKeyError
from qaseio.exceptions import ApiAttributeError
from qaseio.exceptions import ApiException

# import models into sdk package
from qaseio.models.attachment import Attachment
from qaseio.models.attachment_get import AttachmentGet
from qaseio.models.attachment_hash import AttachmentHash
from qaseio.models.attachment_list_response import AttachmentListResponse
from qaseio.models.attachment_list_response_all_of_result import AttachmentListResponseAllOfResult
from qaseio.models.attachment_response import AttachmentResponse
from qaseio.models.attachment_uploads_response import AttachmentUploadsResponse
from qaseio.models.author import Author
from qaseio.models.author_list_response import AuthorListResponse
from qaseio.models.author_list_response_all_of_result import AuthorListResponseAllOfResult
from qaseio.models.author_response import AuthorResponse
from qaseio.models.base_error_field_response import BaseErrorFieldResponse
from qaseio.models.base_error_field_response_error_fields_inner import BaseErrorFieldResponseErrorFieldsInner
from qaseio.models.base_error_response import BaseErrorResponse
from qaseio.models.base_response import BaseResponse
from qaseio.models.bulk200_response import Bulk200Response
from qaseio.models.bulk200_response_all_of_result import Bulk200ResponseAllOfResult
from qaseio.models.create_results_request_v2 import CreateResultsRequestV2
from qaseio.models.create_results_v2422_response import CreateResultsV2422Response
from qaseio.models.custom_field import CustomField
from qaseio.models.custom_field_create import CustomFieldCreate
from qaseio.models.custom_field_create_value_inner import CustomFieldCreateValueInner
from qaseio.models.custom_field_list_response import CustomFieldListResponse
from qaseio.models.custom_field_response import CustomFieldResponse
from qaseio.models.custom_field_update import CustomFieldUpdate
from qaseio.models.custom_field_value import CustomFieldValue
from qaseio.models.custom_fields_response import CustomFieldsResponse
from qaseio.models.custom_fields_response_all_of_result import CustomFieldsResponseAllOfResult
from qaseio.models.defect import Defect
from qaseio.models.defect_create import DefectCreate
from qaseio.models.defect_list_response import DefectListResponse
from qaseio.models.defect_list_response_all_of_result import DefectListResponseAllOfResult
from qaseio.models.defect_query import DefectQuery
from qaseio.models.defect_response import DefectResponse
from qaseio.models.defect_status import DefectStatus
from qaseio.models.defect_update import DefectUpdate
from qaseio.models.environment import Environment
from qaseio.models.environment_create import EnvironmentCreate
from qaseio.models.environment_list_response import EnvironmentListResponse
from qaseio.models.environment_list_response_all_of_result import EnvironmentListResponseAllOfResult
from qaseio.models.environment_response import EnvironmentResponse
from qaseio.models.environment_update import EnvironmentUpdate
from qaseio.models.external_issue import ExternalIssue
from qaseio.models.external_issue_issues_inner import ExternalIssueIssuesInner
from qaseio.models.hash_response import HashResponse
from qaseio.models.hash_response_all_of_result import HashResponseAllOfResult
from qaseio.models.id_response import IdResponse
from qaseio.models.id_response_all_of_result import IdResponseAllOfResult
from qaseio.models.milestone import Milestone
from qaseio.models.milestone_create import MilestoneCreate
from qaseio.models.milestone_list_response import MilestoneListResponse
from qaseio.models.milestone_list_response_all_of_result import MilestoneListResponseAllOfResult
from qaseio.models.milestone_response import MilestoneResponse
from qaseio.models.milestone_update import MilestoneUpdate
from qaseio.models.plan import Plan
from qaseio.models.plan_create import PlanCreate
from qaseio.models.plan_detailed import PlanDetailed
from qaseio.models.plan_detailed_all_of_cases import PlanDetailedAllOfCases
from qaseio.models.plan_list_response import PlanListResponse
from qaseio.models.plan_list_response_all_of_result import PlanListResponseAllOfResult
from qaseio.models.plan_query import PlanQuery
from qaseio.models.plan_response import PlanResponse
from qaseio.models.plan_update import PlanUpdate
from qaseio.models.project import Project
from qaseio.models.project_access import ProjectAccess
from qaseio.models.project_code_response import ProjectCodeResponse
from qaseio.models.project_code_response_all_of_result import ProjectCodeResponseAllOfResult
from qaseio.models.project_counts import ProjectCounts
from qaseio.models.project_counts_defects import ProjectCountsDefects
from qaseio.models.project_counts_runs import ProjectCountsRuns
from qaseio.models.project_create import ProjectCreate
from qaseio.models.project_list_response import ProjectListResponse
from qaseio.models.project_list_response_all_of_result import ProjectListResponseAllOfResult
from qaseio.models.project_response import ProjectResponse
from qaseio.models.qql_defect import QqlDefect
from qaseio.models.qql_plan import QqlPlan
from qaseio.models.qql_test_case import QqlTestCase
from qaseio.models.relation_suite import RelationSuite
from qaseio.models.relation_suite_item import RelationSuiteItem
from qaseio.models.requirement import Requirement
from qaseio.models.response import Response
from qaseio.models.result import Result
from qaseio.models.result_attachment import ResultAttachment
from qaseio.models.result_create import ResultCreate
from qaseio.models.result_create_bulk import ResultCreateBulk
from qaseio.models.result_create_case import ResultCreateCase
from qaseio.models.result_create_response import ResultCreateResponse
from qaseio.models.result_create_response_all_of_result import ResultCreateResponseAllOfResult
from qaseio.models.result_create_v2 import ResultCreateV2
from qaseio.models.result_execution import ResultExecution
from qaseio.models.result_list_response import ResultListResponse
from qaseio.models.result_list_response_all_of_result import ResultListResponseAllOfResult
from qaseio.models.result_relations import ResultRelations
from qaseio.models.result_response import ResultResponse
from qaseio.models.result_step import ResultStep
from qaseio.models.result_step_data import ResultStepData
from qaseio.models.result_step_execution import ResultStepExecution
from qaseio.models.result_update import ResultUpdate
from qaseio.models.resultcreate_bulk import ResultcreateBulk
from qaseio.models.run import Run
from qaseio.models.run_create import RunCreate
from qaseio.models.run_environment import RunEnvironment
from qaseio.models.run_list_response import RunListResponse
from qaseio.models.run_list_response_all_of_result import RunListResponseAllOfResult
from qaseio.models.run_milestone import RunMilestone
from qaseio.models.run_public import RunPublic
from qaseio.models.run_public_response import RunPublicResponse
from qaseio.models.run_public_response_all_of_result import RunPublicResponseAllOfResult
from qaseio.models.run_response import RunResponse
from qaseio.models.run_stats import RunStats
from qaseio.models.search_response import SearchResponse
from qaseio.models.search_response_all_of_result import SearchResponseAllOfResult
from qaseio.models.search_response_all_of_result_entities import SearchResponseAllOfResultEntities
from qaseio.models.shared_step import SharedStep
from qaseio.models.shared_step_content import SharedStepContent
from qaseio.models.shared_step_content_create import SharedStepContentCreate
from qaseio.models.shared_step_create import SharedStepCreate
from qaseio.models.shared_step_list_response import SharedStepListResponse
from qaseio.models.shared_step_list_response_all_of_result import SharedStepListResponseAllOfResult
from qaseio.models.shared_step_response import SharedStepResponse
from qaseio.models.shared_step_update import SharedStepUpdate
from qaseio.models.suite import Suite
from qaseio.models.suite_create import SuiteCreate
from qaseio.models.suite_delete import SuiteDelete
from qaseio.models.suite_list_response import SuiteListResponse
from qaseio.models.suite_list_response_all_of_result import SuiteListResponseAllOfResult
from qaseio.models.suite_response import SuiteResponse
from qaseio.models.suite_update import SuiteUpdate
from qaseio.models.system_field import SystemField
from qaseio.models.system_field_list_response import SystemFieldListResponse
from qaseio.models.system_field_option import SystemFieldOption
from qaseio.models.tag_value import TagValue
from qaseio.models.test_case import TestCase
from qaseio.models.test_case_create import TestCaseCreate
from qaseio.models.test_case_external_issues import TestCaseExternalIssues
from qaseio.models.test_case_external_issues_links_inner import TestCaseExternalIssuesLinksInner
from qaseio.models.test_case_list_response import TestCaseListResponse
from qaseio.models.test_case_list_response_all_of_result import TestCaseListResponseAllOfResult
from qaseio.models.test_case_params import TestCaseParams
from qaseio.models.test_case_query import TestCaseQuery
from qaseio.models.test_case_response import TestCaseResponse
from qaseio.models.test_case_update import TestCaseUpdate
from qaseio.models.test_casebulk import TestCasebulk
from qaseio.models.test_casebulk_cases_inner import TestCasebulkCasesInner
from qaseio.models.test_caseexternal_issues import TestCaseexternalIssues
from qaseio.models.test_step import TestStep
from qaseio.models.test_step_create import TestStepCreate
from qaseio.models.test_step_result import TestStepResult
from qaseio.models.test_step_result_create import TestStepResultCreate



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.result_step import ResultStep

class TestResultStep(unittest.TestCase):
    """ResultStep unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ResultStep:
        """Test ResultStep
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ResultStep`
        """
        model = ResultStep()
        if include_optional:
            return ResultStep(
                id = '',
                step_type = '',
                data = qaseio.models.result_step_data.ResultStepData(
                    action = '', 
                    expected_result = '', ),
                parent_id = '',
                execution = qaseio.models.result_step_execution.ResultStepExecution(
                    start_time = 1.337, 
                    end_time = 1.337, 
                    status = '', 
                    duration = 56, ),
                attachments = [
                    qaseio.models.result_attachment.ResultAttachment(
                        id = '', 
                        file_name = '', 
                        mime_type = '', 
                        file_path = '', 
                        content = '', 
                        size = 56, )
                    ],
                steps = [
                    None
                    ]
            )
        else:
            return ResultStep(
        )
        """

    def testResultStep(self):
        """Test ResultStep"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.result_create_case import ResultCreateCase

class TestResultCreateCase(unittest.TestCase):
    """ResultCreateCase unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ResultCreateCase:
        """Test ResultCreateCase
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ResultCreateCase`
        """
        model = ResultCreateCase()
        if include_optional:
            return ResultCreateCase(
                title = '',
                suite_title = '',
                description = '',
                preconditions = '',
                postconditions = '',
                layer = '',
                severity = '',
                priority = ''
            )
        else:
            return ResultCreateCase(
        )
        """

    def testResultCreateCase(self):
        """Test ResultCreateCase"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.run_list_response import RunListResponse

class TestRunListResponse(unittest.TestCase):
    """RunListResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> RunListResponse:
        """Test RunListResponse
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `RunListResponse`
        """
        model = RunListResponse()
        if include_optional:
            return RunListResponse(
                status = True,
                result = qaseio.models.run_list_response_all_of_result.RunListResponse_allOf_result(
                    total = 56, 
                    filtered = 56, 
                    count = 56, 
                    entities = [
                        qaseio.models.run.Run(
                            id = 56, 
                            title = '', 
                            description = '', 
                            status = 56, 
                            status_text = '', 
                            start_time = '2021-12-30T19:23:59Z', 
                            end_time = '2021-12-30T19:23:59Z', 
                            public = True, 
                            stats = qaseio.models.run_stats.Run_stats(
                                total = 56, 
                                untested = 56, 
                                passed = 56, 
                                failed = 56, 
                                blocked = 56, 
                                skipped = 56, 
                                retest = 56, 
                                in_progress = 56, 
                                invalid = 56, ), 
                            time_spent = 56, 
                            environment = qaseio.models.run_environment.Run_environment(
                                title = '', 
                                description = '', 
                                slug = '', 
                                host = '', ), 
                            milestone = qaseio.models.run_milestone.Run_milestone(
                                title = '', 
                                description = '', ), 
                            custom_fields = [
                                qaseio.models.custom_field_value.CustomFieldValue(
                                    id = 56, 
                                    value = '', )
                                ], 
                            tags = [
                                qaseio.models.tag_value.TagValue(
                                    title = '', 
                                    internal_id = 56, )
                                ], 
                            cases = [
                                56
                                ], )
                        ], )
            )
        else:
            return RunListResponse(
        )
        """

    def testRunListResponse(self):
        """Test RunListResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from qaseio.api.results_api import ResultsApi


class TestResultsApi(unittest.TestCase):
    """ResultsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ResultsApi()

    def tearDown(self) -> None:
        pass

    def test_create_result(self) -> None:
        """Test case for create_result

        Create test run result.
        """
        pass

    def test_create_result_bulk(self) -> None:
        """Test case for create_result_bulk

        Bulk create test run result.
        """
        pass

    def test_delete_result(self) -> None:
        """Test case for delete_result

        Delete test run result.
        """
        pass

    def test_get_result(self) -> None:
        """Test case for get_result

        Get test run result by code.
        """
        pass

    def test_get_results(self) -> None:
        """Test case for get_results

        Get all test run results.
        """
        pass

    def test_update_result(self) -> None:
        """Test case for update_result

        Update test run result.
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.create_results_request_v2 import CreateResultsRequestV2

class TestCreateResultsRequestV2(unittest.TestCase):
    """CreateResultsRequestV2 unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> CreateResultsRequestV2:
        """Test CreateResultsRequestV2
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `CreateResultsRequestV2`
        """
        model = CreateResultsRequestV2()
        if include_optional:
            return CreateResultsRequestV2(
                results = [
                    qaseio.models.result_create_v2.ResultCreateV2(
                        id = '', 
                        title = '', 
                        signature = '', 
                        testops_id = 56, 
                        execution = qaseio.models.result_execution.ResultExecution(
                            start_time = 1.337, 
                            end_time = 1.337, 
                            status = '', 
                            duration = 56, 
                            stacktrace = '', 
                            thread = '', ), 
                        fields = {
                            'key' : ''
                            }, 
                        attachments = [
                            qaseio.models.result_attachment.ResultAttachment(
                                id = '', 
                                file_name = '', 
                                mime_type = '', 
                                file_path = '', 
                                content = '', 
                                size = 56, )
                            ], 
                        steps = [
                            qaseio.models.result_step.ResultStep(
                                id = '', 
                                step_type = '', 
                                data = qaseio.models.result_step_data.ResultStepData(
                                    action = '', 
                                    expected_result = '', ), 
                                parent_id = '', )
                            ], 
                        params = {
                            'key' : ''
                            }, 
                        author = '', 
                        relations = qaseio.models.relation.Relation(
                            suite = qaseio.models.relation_suite.RelationSuite(
                                data = [
                                    qaseio.models.relation_suite_item.RelationSuiteItem(
                                        title = '', 
                                        public_id = 56, )
                                    ], ), ), 
                        muted = True, 
                        message = '', 
                        created_at = 1.337, )
                    ]
            )
        else:
            return CreateResultsRequestV2(
        )
        """

    def testCreateResultsRequestV2(self):
        """Test CreateResultsRequestV2"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from qaseio.api.environments_api import EnvironmentsApi


class TestEnvironmentsApi(unittest.TestCase):
    """EnvironmentsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = EnvironmentsApi()

    def tearDown(self) -> None:
        pass

    def test_create_environment(self) -> None:
        """Test case for create_environment

        Create a new environment.
        """
        pass

    def test_delete_environment(self) -> None:
        """Test case for delete_environment

        Delete environment.
        """
        pass

    def test_get_environment(self) -> None:
        """Test case for get_environment

        Get a specific environment.
        """
        pass

    def test_get_environments(self) -> None:
        """Test case for get_environments

        Get all environments.
        """
        pass

    def test_update_environment(self) -> None:
        """Test case for update_environment

        Update environment.
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.test_case_list_response_all_of_result import TestCaseListResponseAllOfResult

class TestTestCaseListResponseAllOfResult(unittest.TestCase):
    """TestCaseListResponseAllOfResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> TestCaseListResponseAllOfResult:
        """Test TestCaseListResponseAllOfResult
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `TestCaseListResponseAllOfResult`
        """
        model = TestCaseListResponseAllOfResult()
        if include_optional:
            return TestCaseListResponseAllOfResult(
                total = 56,
                filtered = 56,
                count = 56,
                entities = [
                    qaseio.models.test_case.TestCase(
                        id = 56, 
                        position = 56, 
                        title = '', 
                        description = '', 
                        preconditions = '', 
                        postconditions = '', 
                        severity = 56, 
                        priority = 56, 
                        type = 56, 
                        layer = 56, 
                        is_flaky = 56, 
                        behavior = 56, 
                        automation = 56, 
                        status = 56, 
                        milestone_id = 56, 
                        suite_id = 56, 
                        custom_fields = [
                            qaseio.models.custom_field_value.CustomFieldValue(
                                id = 56, 
                                value = '', )
                            ], 
                        attachments = [
                            qaseio.models.attachment.Attachment(
                                size = 56, 
                                mime = '', 
                                filename = '', 
                                url = '', )
                            ], 
                        steps_type = '', 
                        steps = [
                            qaseio.models.test_step.TestStep(
                                hash = '', 
                                shared_step_hash = '', 
                                shared_step_nested_hash = '', 
                                position = 56, 
                                action = '', 
                                expected_result = '', 
                                data = '', )
                            ], 
                        params = null, 
                        tags = [
                            qaseio.models.tag_value.TagValue(
                                title = '', 
                                internal_id = 56, )
                            ], 
                        member_id = 56, 
                        author_id = 56, 
                        created_at = '2021-12-30T19:23:59Z', 
                        updated_at = '2021-12-30T19:23:59Z', 
                        deleted = '2021-12-30T19:23:59.000000Z', 
                        created = '2021-12-30T19:23:59.000000Z', 
                        updated = '2021-12-30T19:23:59.000000Z', 
                        external_issues = [
                            qaseio.models.external_issue.ExternalIssue(
                                type = '', 
                                issues = [
                                    qaseio.models.external_issue_issues_inner.ExternalIssue_issues_inner(
                                        id = '', 
                                        link = '', )
                                    ], )
                            ], )
                    ]
            )
        else:
            return TestCaseListResponseAllOfResult(
        )
        """

    def testTestCaseListResponseAllOfResult(self):
        """Test TestCaseListResponseAllOfResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictBool, StrictInt, StrictStr
from pydantic import Field
from typing_extensions import Annotated
from qaseio.models.result_create_case import ResultCreateCase
from qaseio.models.test_step_result_create import TestStepResultCreate
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class ResultCreate(BaseModel):
    """
    ResultCreate
    """ # noqa: E501
    case_id: Optional[StrictInt] = None
    case: Optional[ResultCreateCase] = None
    status: StrictStr = Field(description="Can have the following values `passed`, `failed`, `blocked`, `skipped`, `invalid` + custom statuses")
    start_time: Optional[Annotated[int, Field(strict=True, ge=0)]] = None
    time: Optional[Annotated[int, Field(le=31536000, strict=True, ge=0)]] = None
    time_ms: Optional[Annotated[int, Field(le=31536000000, strict=True, ge=0)]] = None
    defect: Optional[StrictBool] = None
    attachments: Optional[List[StrictStr]] = None
    stacktrace: Optional[StrictStr] = None
    comment: Optional[StrictStr] = None
    param: Optional[Dict[str, StrictStr]] = Field(default=None, description="A map of parameters (name => value)")
    steps: Optional[List[TestStepResultCreate]] = None
    author_id: Optional[StrictInt] = None
    __properties: ClassVar[List[str]] = ["case_id", "case", "status", "start_time", "time", "time_ms", "defect", "attachments", "stacktrace", "comment", "param", "steps", "author_id"]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ResultCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of case
        if self.case:
            _dict['case'] = self.case.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in steps (list)
        _items = []
        if self.steps:
            for _item in self.steps:
                if _item:
                    _items.append(_item.to_dict())
            _dict['steps'] = _items
        # set to None if start_time (nullable) is None
        # and model_fields_set contains the field
        if self.start_time is None and "start_time" in self.model_fields_set:
            _dict['start_time'] = None

        # set to None if time (nullable) is None
        # and model_fields_set contains the field
        if self.time is None and "time" in self.model_fields_set:
            _dict['time'] = None

        # set to None if time_ms (nullable) is None
        # and model_fields_set contains the field
        if self.time_ms is None and "time_ms" in self.model_fields_set:
            _dict['time_ms'] = None

        # set to None if defect (nullable) is None
        # and model_fields_set contains the field
        if self.defect is None and "defect" in self.model_fields_set:
            _dict['defect'] = None

        # set to None if attachments (nullable) is None
        # and model_fields_set contains the field
        if self.attachments is None and "attachments" in self.model_fields_set:
            _dict['attachments'] = None

        # set to None if stacktrace (nullable) is None
        # and model_fields_set contains the field
        if self.stacktrace is None and "stacktrace" in self.model_fields_set:
            _dict['stacktrace'] = None

        # set to None if comment (nullable) is None
        # and model_fields_set contains the field
        if self.comment is None and "comment" in self.model_fields_set:
            _dict['comment'] = None

        # set to None if param (nullable) is None
        # and model_fields_set contains the field
        if self.param is None and "param" in self.model_fields_set:
            _dict['param'] = None

        # set to None if steps (nullable) is None
        # and model_fields_set contains the field
        if self.steps is None and "steps" in self.model_fields_set:
            _dict['steps'] = None

        # set to None if author_id (nullable) is None
        # and model_fields_set contains the field
        if self.author_id is None and "author_id" in self.model_fields_set:
            _dict['author_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ResultCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "case_id": obj.get("case_id"),
            "case": ResultCreateCase.from_dict(obj.get("case")) if obj.get("case") is not None else None,
            "status": obj.get("status"),
            "start_time": obj.get("start_time"),
            "time": obj.get("time"),
            "time_ms": obj.get("time_ms"),
            "defect": obj.get("defect"),
            "attachments": obj.get("attachments"),
            "stacktrace": obj.get("stacktrace"),
            "comment": obj.get("comment"),
            "param": obj.get("param"),
            "steps": [TestStepResultCreate.from_dict(_item) for _item in obj.get("steps")] if obj.get("steps") is not None else None,
            "author_id": obj.get("author_id")
        })
        return _obj





# flake8: noqa

# import apis into api package
from qaseio.api.attachments_api import AttachmentsApi
from qaseio.api.authors_api import AuthorsApi
from qaseio.api.cases_api import CasesApi
from qaseio.api.custom_fields_api import CustomFieldsApi
from qaseio.api.defects_api import DefectsApi
from qaseio.api.environments_api import EnvironmentsApi
from qaseio.api.milestones_api import MilestonesApi
from qaseio.api.plans_api import PlansApi
from qaseio.api.projects_api import ProjectsApi
from qaseio.api.result_api import ResultApi
from qaseio.api.results_api import ResultsApi
from qaseio.api.runs_api import RunsApi
from qaseio.api.search_api import SearchApi
from qaseio.api.shared_steps_api import SharedStepsApi
from qaseio.api.suites_api import SuitesApi
from qaseio.api.system_fields_api import SystemFieldsApi




# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from qaseio.api.system_fields_api import SystemFieldsApi


class TestSystemFieldsApi(unittest.TestCase):
    """SystemFieldsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = SystemFieldsApi()

    def tearDown(self) -> None:
        pass

    def test_get_system_fields(self) -> None:
        """Test case for get_system_fields

        Get all System Fields.
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.test_case_create import TestCaseCreate

class TestTestCaseCreate(unittest.TestCase):
    """TestCaseCreate unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> TestCaseCreate:
        """Test TestCaseCreate
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `TestCaseCreate`
        """
        model = TestCaseCreate()
        if include_optional:
            return TestCaseCreate(
                description = '',
                preconditions = '',
                postconditions = '',
                title = '',
                severity = 56,
                priority = 56,
                behavior = 56,
                type = 56,
                layer = 56,
                is_flaky = 56,
                suite_id = 56,
                milestone_id = 56,
                automation = 56,
                status = 56,
                attachments = [
                    ''
                    ],
                steps = [
                    qaseio.models.test_step_create.TestStepCreate(
                        action = '', 
                        expected_result = '', 
                        data = '', 
                        position = 56, 
                        attachments = [
                            ''
                            ], 
                        steps = [
                            None
                            ], )
                    ],
                tags = [
                    ''
                    ],
                params = {
                    'key' : [
                        ''
                        ]
                    },
                custom_field = {
                    'key' : ''
                    },
                created_at = '',
                updated_at = ''
            )
        else:
            return TestCaseCreate(
                title = '',
        )
        """

    def testTestCaseCreate(self):
        """Test TestCaseCreate"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List, Optional
from pydantic import BaseModel, StrictStr, field_validator
from pydantic import Field
from typing_extensions import Annotated
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class ProjectCreate(BaseModel):
    """
    ProjectCreate
    """ # noqa: E501
    title: StrictStr = Field(description="Project title.")
    code: Annotated[str, Field(strict=True)] = Field(description="Project code. Unique for team. Digits and special characters are not allowed.")
    description: Optional[StrictStr] = Field(default=None, description="Project description.")
    access: Optional[StrictStr] = None
    group: Optional[StrictStr] = Field(default=None, description="Team group hash. Required if access param is set to group.")
    __properties: ClassVar[List[str]] = ["title", "code", "description", "access", "group"]

    @field_validator('code')
    def code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z]{2,10}$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z]{2,10}$/")
        return value

    @field_validator('access')
    def access_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('all', 'group', 'none'):
            raise ValueError("must be one of enum values ('all', 'group', 'none')")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ProjectCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ProjectCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "title": obj.get("title"),
            "code": obj.get("code"),
            "description": obj.get("description"),
            "access": obj.get("access"),
            "group": obj.get("group")
        })
        return _obj





# coding: utf-8

"""
    Qase.io TestOps API

    Qase TestOps API Specification.

    The version of the OpenAPI document: 1.0.0
    Contact: support@qase.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from qaseio.models.result_create_v2 import ResultCreateV2

class TestResultCreateV2(unittest.TestCase):
    """ResultCreateV2 unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ResultCreateV2:
        """Test ResultCreateV2
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ResultCreateV2`
        """
        model = ResultCreateV2()
        if include_optional:
            return ResultCreateV2(
                id = '',
                title = '',
                signature = '',
                testops_id = 56,
                execution = qaseio.models.result_execution.ResultExecution(
                    start_time = 1.337, 
                    end_time = 1.337, 
                    status = '', 
                    duration = 56, 
                    stacktrace = '', 
                    thread = '', ),
                fields = {
                    'key' : ''
                    },
                attachments = [
                    qaseio.models.result_attachment.ResultAttachment(
                        id = '', 
                        file_name = '', 
                        mime_type = '', 
                        file_path = '', 
                        content = '', 
                        size = 56, )
                    ],
                steps = [
                    qaseio.models.result_step.ResultStep(
                        id = '', 
                        step_type = '', 
                        data = qaseio.models.result_step_data.ResultStepData(
                            action = '', 
                            expected_result = '', ), 
                        parent_id = '', 
                        execution = qaseio.models.result_step_execution.ResultStepExecution(
                            start_time = 1.337, 
                            end_time = 1.337, 
                            status = '', 
                            duration = 56, ), 
                        attachments = [
                            qaseio.models.result_attachment.ResultAttachment(
                                id = '', 
                                file_name = '', 
                                mime_type = '', 
                                file_path = '', 
                                content = '', 
                                size = 56, )
                            ], 
                        steps = [
                            None
                            ], )
                    ],
                params = {
                    'key' : ''
                    },
                author = '',
                relations = qaseio.models.relation.Relation(
                    suite = qaseio.models.relation_suite.RelationSuite(
                        data = [
                            qaseio.models.relation_suite_item.RelationSuiteItem(
                                title = '', 
                                public_id = 56, )
                            ], ), ),
                muted = True,
                message = '',
                created_at = 1.337
            )
        else:
            return ResultCreateV2(
                title = '',
                execution = qaseio.models.result_execution.ResultExecution(
                    start_time = 1.337, 
                    end_time = 1.337, 
                    status = '', 
                    duration = 56, 
                    stacktrace = '', 
                    thread = '', ),
        )
        """

    def testResultCreateV2(self):
        """Test ResultCreateV2"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
